#include "drawing.h"

#include "global.h"
#include "components.h"
#include "map.h"


/*
 * draws a rectangle of the specified size, automatically splitting it up
 * into smaller ones using the global "quality" variable
 * Only the width and height of the size will be used
 */
void drawRectangle(Size size) {

    int xPieces, yPieces; // pieces of rectangles by the 2 axes
	GLfloat xSize, ySize; // sizes of pieces by the 2 axes
	GLfloat xPos, yPos; // position of the center of the current rectangle
	
	int i, j;
	
	xPieces = (int)(size.width * quality / QUALITY_DIV_STACK) + 1;
	yPieces = (int)(size.height * quality / QUALITY_DIV_STACK) + 1;
	
	xSize = size.width / (GLfloat)xPieces;
	ySize = size.height / (GLfloat)yPieces;
	
#ifdef DEBUG_RECTANGLESPLIT
    printf("size.width = %f, size.height = %f, quality = %f, xPieces = %d, yPieces = %d, xSize = %f, ySize = %f\n\n", size.width, size.height, quality, xPieces, yPieces, xSize, ySize);
#endif
	
	xPos = -(size.width / 2.0) + xSize / 2.0;
	
	for(i = 0; i < xPieces; i++, xPos += xSize) {
	
	    yPos = -(size.height / 2.0) + ySize / 2.0;
	
	    for(j = 0; j < yPieces; j++, yPos += ySize) {
	        
	        glBegin(GL_QUADS);
		        glVertex3f(xPos-xSize/2.0, yPos-ySize/2.0, 0.0); // a
		        glVertex3f(xPos+xSize/2.0, yPos-ySize/2.0, 0.0); // b
		        glVertex3f(xPos+xSize/2.0, yPos+ySize/2.0, 0.0); // c
		        glVertex3f(xPos-xSize/2.0, yPos+ySize/2.0, 0.0); // d
	        glEnd();
	        
	    }
	}
	
}


/* INTERFACE
 * draws a cuboid of the specified size
 */
void drawCuboid(Size size) {
	
	Size s;
	
	glPushMatrix(); // A
	    glTranslatef(0.0, 0.0, +size.length/2.0);
	    
	    s.width = size.width;
	    s.height = size.height;
	    
	    drawRectangle(s);
	glPopMatrix();
	
	glPushMatrix(); // B
	    glTranslatef(+size.width/2.0, 0.0, 0.0);
	    glRotatef(90.0, 0.0, 1.0, 0.0);
	    
	    s.width = size.length;
	    s.height = size.height;
	    
	    drawRectangle(s);
	glPopMatrix();
	
	glPushMatrix(); // E
	    glTranslatef(0.0, 0.0, -size.length/2.0);
	    glRotatef(180.0, 0.0, 1.0, 0.0);
	    
	    s.width = size.width;
	    s.height = size.height;
	    
	    drawRectangle(s);
	glPopMatrix();
	
	glPushMatrix(); // D
	    glTranslatef(-size.width/2.0, 0.0, 0.0);
	    glRotatef(-90.0, 0.0, 1.0, 0.0);
	    
	    s.width = size.length;
	    s.height = size.height;
	    
	    drawRectangle(s);
	glPopMatrix();
	
	glPushMatrix(); // F
	    glTranslatef(0.0, -size.height/2.0, 0.0);
	    glRotatef(-90.0, 1.0, 0.0, 0.0);
	    
	    s.width = size.width;
	    s.height = size.length;
	    
	    drawRectangle(s);
	glPopMatrix();
	
	glPushMatrix(); // C
	    glTranslatef(0.0, +size.height/2.0, 0.0);
	    glRotatef(+90.0, 1.0, 0.0, 0.0);
	    
	    s.width = size.width;
	    s.height = size.length;
	    
	    drawRectangle(s);
	glPopMatrix();
	
}

/* INTERFACE
 */
void drawSphere(Size size, int stacks, int slices) {

	int i, j;
	
	stacks *= quality;
	slices *= quality;
	
	if(stacks < 4) stacks = 4;
	if(slices < 4) slices = 4;
	
	glPushMatrix();
	
		glScalef(size.width/2.0, size.height/2.0, size.length/2.0);
	
		for(i = 0; i <= stacks; i++) {
			double stack0 = GL_PI * (-0.5 + (double)(i - 1) / (double)stacks);
			double z0  = sin(stack0);
			double zr0 =  cos(stack0);
		
			double stack1 = GL_PI * (-0.5 + (double)i / (double)stacks);
			double z1 = sin(stack1);
			double zr1 = cos(stack1);
		
			glBegin(GL_QUAD_STRIP);
			
				for(j = 0; j <= slices; j++) {
					double slice = 2 * GL_PI * (double)(j - 1) / (double)slices;
					double x = cos(slice);
					double y = sin(slice);
				
					glVertex3f(x * zr0, y * zr0, z0);
					glVertex3f(x * zr1, y * zr1, z1);
				}
				
			glEnd();
		}
		
	glPopMatrix();
}

/* INTERFACE
 */
void drawCylinder(Size size, int sides) {
	
	GLfloat x, z, angle;
    int i, j;
	
	int stacks = quality * size.height / QUALITY_DIV_STACK;
	GLfloat stackHeight = 1.0 / (float)stacks;
	
	if(stacks < 1) {
		stacks = 1;
		stackHeight = 1.0;
	}
	
	sides *= quality;
	
	if(sides < 4) sides = 4;
    
	glPushMatrix();
	
		glScalef(size.width, size.height, size.length);
	
		// lower base circle
		glBegin( GL_TRIANGLE_FAN );
		
			glVertex3f( 0.0, -0.5, 0.0 );
			
			for( i = 0, angle = 0.0; i <= sides; i++, angle += ( GL_PI / ( sides / 2.0 ) ) ) {
			
				x = 0.5 * cos( angle );
				z = 0.5 * sin( angle );				
				glVertex3f( x, -0.5, z );			
			}
			
		glEnd();
				
		// side
		for(j = 0; j < stacks; j++) { // this method is a waste of resources, but no time to optimize :D
			glBegin( GL_QUAD_STRIP );
				
				for( i = 0, angle = 0.0; i <= sides; i++, angle += ( GL_PI / ( sides / 2.0 ) ) ) {
				
					x = 0.5 * cos( angle );
					z = 0.5 * sin( angle );				
					glVertex3f( x,  j   * stackHeight - 0.5, z );
					glVertex3f( x, (j+1)* stackHeight - 0.5, z );			
				}
			
			glEnd();
		}
		// upper base circle
		glBegin( GL_TRIANGLE_FAN );
		
			glVertex3f( 0.0, +0.5, 0.0 );
			
			for( i = 0, angle = ( 2.0 * GL_PI ); i <= sides; i++, angle -= ( GL_PI / ( sides / 2.0 ) ) ) {
			
				x = 0.5 * cos( angle );
				z = 0.5 * sin( angle );				
				glVertex3f( x, +0.5, z );			
			}
			
		glEnd();
	
	glPopMatrix();
	
}

/* INTERFACE
 */
void drawArch(Size size, int sides) {
	
	GLfloat x, y, x1, y1, angle;
	int i;
	
	sides *= quality;
	
	if(sides < 4) sides = 4;
	
	GLfloat * angles = malloc((sides+1) * sizeof(GLfloat));
	
	glPushMatrix();
	
		glTranslatef(0.0, -size.height/2.0, 0.0);
		glScalef(size.width/2.0, size.height, size.length);
		
		glBegin( GL_QUAD_STRIP );
			
			for( i = 0, angle = 0.0; i <= sides; i++, angle += ( GL_PI / (float)sides ) ) {
			
				x = cos( angle );
				y = sin( angle );				
				glVertex3f( x, y, -0.5 );			
				glVertex3f( x, y, +0.5 );	
				
				angles[i] = angle;
			}
			
		glEnd();
		
		x1 = cos( angles[0] );
		y1 = sin( angles[0] );
		
		for(i = 0; i < sides; i++) {			
				x = x1;
				y = y1;
				x1 = cos( angles[i+1] );
				y1 = sin( angles[i+1] );
			glBegin(GL_QUADS);
				glVertex3f( x,   y,   -0.5 );
				glVertex3f( x1,  y1,  -0.5 );
				glVertex3f( x1, +1.0, -0.5 );
				glVertex3f( x,  +1.0, -0.5 );
			glEnd();
			glBegin(GL_QUADS);
				glVertex3f( x,  +1.0, +0.5 );
				glVertex3f( x1, +1.0, +0.5 );
				glVertex3f( x1,  y1,  +0.5 );
				glVertex3f( x,   y,   +0.5 );
			glEnd();
		}
		
	glPopMatrix();
	
	free(angles);
}

/* INTERFACE
 * draws a torus with rectangular side. So it looks like a pipe or ring instead of a doughnut
 */
void drawCubicTorus(Size size, GLfloat thickness, int sides) {

	GLfloat x, z, angle, R = 0.5, r = 0.5 - thickness / size.width;
    int i, j;
	
	int stacks = quality * size.height / QUALITY_DIV_STACK;
	GLfloat stackHeight = 1.0 / (float)stacks;
	
	if(stacks < 1) {
		stacks = 1;
		stackHeight = 1.0;
	}
	
	sides *= quality;
	
	if(sides < 4) sides = 4;
    
	glPushMatrix();
	
		glScalef(size.width, size.height, size.length);
	
		// lower base circle
		glBegin( GL_QUAD_STRIP );
			
			for( i = 0, angle = 0.0; i <= sides; i++, angle += ( GL_PI / ( sides / 2.0 ) ) ) {
				
				x = r * cos( angle );
				z = r * sin( angle );					
				glVertex3f( x, -0.5, z );

				x = R * cos( angle );
				z = R * sin( angle );	
				glVertex3f( x, -0.5, z );							
			}
			
		glEnd();
				
		// outer rim
		for(j = 0; j < stacks; j++) { // this method is a waste of resources, but no time to optimize :D
			glBegin( GL_QUAD_STRIP );
				
				for( i = 0, angle = 0.0; i <= sides; i++, angle += ( GL_PI / ( sides / 2.0 ) ) ) {
				
					x = R * cos( angle );
					z = R * sin( angle );				
					glVertex3f( x,  j   * stackHeight - 0.5, z );
					glVertex3f( x, (j+1)* stackHeight - 0.5, z );			
				}
			
			glEnd();
		}
		
		// inner rim
		for(j = 0; j < stacks; j++) { // this method is a waste of resources, but no time to optimize :D
			glBegin( GL_QUAD_STRIP );
				
				for( i = 0, angle = 0.0; i <= sides; i++, angle += ( GL_PI / ( sides / 2.0 ) ) ) {
				
					x = r * cos( angle );
					z = r * sin( angle );				
					glVertex3f( x,  j   * stackHeight - 0.5, z );
					glVertex3f( x, (j+1)* stackHeight - 0.5, z );			
				}
			
			glEnd();
		}
		
		// upper base circle
		glBegin( GL_QUAD_STRIP );
			
			for( i = 0, angle = 0.0; i <= sides; i++, angle += ( GL_PI / ( sides / 2.0 ) ) ) {

				x = R * cos( angle );
				z = R * sin( angle );	
				glVertex3f( x, +0.5, z );

				x = r * cos( angle );
				z = r * sin( angle );					
				glVertex3f( x, +0.5, z );				
			}
			
		glEnd();
	
	glPopMatrix();

}


/* INTERFACE
 */
void drawPigEar(Size size) {

    glPushMatrix();
    
        glScalef(size.width, size.height, size.length);
        
        glBegin( GL_QUADS );
        
            glVertex3f(-0.5, -0.5, -0.5); // d
            glVertex3f(+0.5, -0.5, -0.5); // c
            glVertex3f(+0.5, -0.5, +0.5); // b
            glVertex3f(-0.5, -0.5, +0.5); // a
        
        glEnd();
        
        glBegin( GL_TRIANGLE_FAN );
        
            glVertex3f(+0.0, +0.5, +0.0);
            
            glVertex3f(-0.5, -0.5, +0.5); // a
            glVertex3f(+0.5, -0.5, +0.5); // b
            glVertex3f(+0.5, -0.5, -0.5); // c
            glVertex3f(-0.5, -0.5, -0.5); // d
        
        glEnd();
    
    glPopMatrix();

}

/* INTERFACE
 */
void drawPigTail(Size size) {

	Size s;
	
	glPushMatrix();
	
		glScalef(size.width, size.height, size.length);
		
		s.width = 0.1;
		s.length = 0.1;
		s.height = 1.0;
		
		glPushMatrix();
			glTranslatef(0.0, -0.3, 0.0);
			glRotatef(90.0, 1.0, 0.0, 0.0);
			drawCylinder(s, 6);
		glPopMatrix();
		
		s.width = 0.8;
		s.length = 0.8;
		s.height = 0.1;
		
		glPushMatrix();
			glTranslatef(0.0, +0.05, -0.2);
			glRotatef(90.0, 1.0, -0.4, 0.0);
			drawCubicTorus(s, 0.1, 8);
		glPopMatrix();
		
		glPushMatrix();
			glTranslatef(0.0, +0.05, +0.2);
			glRotatef(90.0, 1.0, -0.4, 0.0);
			drawCubicTorus(s, 0.1, 8);
		glPopMatrix();
		
		s.width = 0.15;
		s.length = 0.3;
		s.height = 0.15;
		
		glPushMatrix();
			glTranslatef(0.0, -0.3, +0.5);
			drawSphere(s, 6, 6);
		glPopMatrix();
		
	glPopMatrix();

}

/* INTERFACE
 */
void drawRoof(Size size) {
	
	glPushMatrix();
	
		glScalef(size.width/1090.0, size.height/100.0, size.length/790.0);
		
		glBegin( GL_TRIANGLES ); // right front triangle
            			
            glVertex3f(+205, -50, +395); 
            glVertex3f(+545, -50, +395);
			glVertex3f(+375, +50, +225); 
        
        glEnd();
		
		glBegin( GL_QUADS ); // right outer side
        
            glVertex3f(+545, -50, +395);
            glVertex3f(+545, -50, -395);
            glVertex3f(+375, +50, -225); 
            glVertex3f(+375, +50, +225); 
        
        glEnd();
		
		glBegin( GL_QUADS ); // right inner side
        
            glVertex3f(+205, -50, -55);
            glVertex3f(+205, -50, +395); 
            glVertex3f(+375, +50, +225); 
            glVertex3f(+375, +50, -225); 
        
        glEnd();
		
		glBegin( GL_TRIANGLES ); // left front triangle
            			
			glVertex3f(-375, +50, +225); 
			glVertex3f(-545, -50, +395);
			glVertex3f(-205, -50, +395); 
        
        glEnd();
		
		glBegin( GL_QUADS ); // left outer side
         
            glVertex3f(-375, +50, +225); 
            glVertex3f(-375, +50, -225);
            glVertex3f(-545, -50, -395);
            glVertex3f(-545, -50, +395);
        
        glEnd();
		
		glBegin( GL_QUADS ); // left inner side
        
            glVertex3f(-375, +50, -225); 
            glVertex3f(-375, +50, +225); 
            glVertex3f(-205, -50, +395); 
            glVertex3f(-205, -50, -55);
        
        glEnd();
		
		glBegin( GL_QUADS ); // rear inner side
        
            glVertex3f(-205, -50, -55);
            glVertex3f(+205, -50, -55);
            glVertex3f(+375, +50, -225); 
            glVertex3f(-375, +50, -225); 
        
        glEnd();
		
		glBegin( GL_QUADS ); // rear outer side
        
            glVertex3f(+545, -50, -395);
            glVertex3f(-545, -50, -395);
            glVertex3f(-375, +50, -225); 
            glVertex3f(+375, +50, -225); 
        
        glEnd();
		
	glPopMatrix();
	
}

/* INTERFACE
 */
void drawWheel(Size size) {
	
	GLfloat angle;
    int i;
	Size s;
	
	glPushMatrix();
	
		glScalef(size.width, size.height, size.length);
		
		s.width = 0.1;
		s.length = 0.1;
		s.height = 1.0;
		
		glPushMatrix();
			glRotatef(90.0, 1.0, 0.0, 0.0);
			drawCylinder(s, 10); // axle
		glPopMatrix();
		
		s.width = 1.0;
		s.length = 1.0;
		s.height = 0.3;
		
		glPushMatrix();
			glRotatef(90.0, 1.0, 0.0, 0.0);
			drawCubicTorus(s, 0.05, 20); // rim
		glPopMatrix();
		
		s.width = 0.03;
		s.length = 0.2;
		s.height = 0.95;
			
		for( i = 0, angle = 0.0; i < 8; i++, angle += 45.0 ) { // spokes		
#ifdef DEBUG_WHEELDRAW
			printf("\n\n===============================\ndrawWheel output:\n\ni: %d, angle:%f\n\n===============================\n\n", i, angle);
#endif
			
			glPushMatrix();
				glRotatef(angle, 0.0, 0.0, 1.0);
				drawCylinder(s, 10);
			glPopMatrix();			
		}
		
	glPopMatrix();
	
}


/* INTERFACE
 * draw bounds of pig movement for testing
 */
void drawBounds() {
	int i, j;
	
	glPushMatrix();
	
		glColor3f(0.0, 0.0, 0.0);
		
		for(i = 0; i < FIELDSIZE/AREASIZE; i++) {
			for(j = 0; j < FIELDSIZE/AREASIZE; j++) {
				if(bounds[i][j] == 0) {
					glBegin(GL_LINE_LOOP);
						glVertex3f(  i      * AREASIZE - FIELDSIZE/2.0, 0.0, (j + 1) * AREASIZE - FIELDSIZE/2.0);
						glVertex3f( (i + 1) * AREASIZE - FIELDSIZE/2.0, 0.0, (j + 1) * AREASIZE - FIELDSIZE/2.0);
						glVertex3f( (i + 1) * AREASIZE - FIELDSIZE/2.0, 0.0,  j      * AREASIZE - FIELDSIZE/2.0);
						glVertex3f(  i      * AREASIZE - FIELDSIZE/2.0, 0.0,  j      * AREASIZE - FIELDSIZE/2.0);
					glEnd();
				} else {
					glBegin(GL_QUADS);
						glVertex3f(  i      * AREASIZE - FIELDSIZE/2.0, 0.0, (j + 1) * AREASIZE - FIELDSIZE/2.0);
						glVertex3f( (i + 1) * AREASIZE - FIELDSIZE/2.0, 0.0, (j + 1) * AREASIZE - FIELDSIZE/2.0);
						glVertex3f( (i + 1) * AREASIZE - FIELDSIZE/2.0, 0.0,  j      * AREASIZE - FIELDSIZE/2.0);
						glVertex3f(  i      * AREASIZE - FIELDSIZE/2.0, 0.0,  j      * AREASIZE - FIELDSIZE/2.0);
					glEnd();
				}
			}
		}
	
	glPopMatrix();
}
